/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Plugin Finder Service.
 *
 * The Initial Developer of the Original Code is IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the IBM Corporation. All Rights Reserved.
 *
 * Contributor(s):
 *  Doron Rosenberg <doronr@us.ibm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

package org.mozilla.pfs;

public class PluginFinderService {

  public org.mozilla.pfs.PluginInfo getPluginInfo(java.lang.String aMimetype, java.lang.String aClientOS, java.lang.String aLocale) {
    org.mozilla.pfs.PluginInfo response = new org.mozilla.pfs.PluginInfo();
    
    if (aMimetype.equals("application/x-shockwave-flash")) {
      response.setPid(1);
      response.setName("Flash Player");
      response.setVersion("7");
      response.setIconUrl("http://goat.austin.ibm.com:8080/flash.gif");
      response.setXPILocation("http://www.nexgenmedia.net/flashlinux/flash-linux.xpi");
      response.setInstallerShowsUI(false);
      response.setManualInstallationURL("");
      response.setLicenseURL("");
    } else if (aMimetype.equals("application/x-mtx")) {
      response.setPid(2);
      response.setName("Viewpoint Media Player");
      response.setVersion("5");
      response.setIconUrl(null);
      response.setXPILocation("http://www.nexgenmedia.net/flashlinux/invalid.xpi");
      response.setInstallerShowsUI(false);
      response.setManualInstallationURL("http://www.viewpoint.com/pub/products/vmp.html");   
      response.setLicenseURL("http://www.viewpoint.com/pub/privacy.html");
    } else {
      response.setPid(-1);
    }
    
    response.setRequestedMimetype(aMimetype);
    return response;
    
  }
}

