/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is CaScadeS, a stylesheet editor for Composer.
 *
 * The Initial Developer of the Original Code is
 * Daniel Glazman.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Original author: Daniel Glazman <daniel@glazman.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
@namespace html url("http://www.w3.org/1999/xhtml");

#sheetInfoTabPanelTitle, .titleLabel {
  font-weight: bold;
  font-size: larger;
}

.bangImportant {
  color: red;
}

html|p#brownFoxLabel {
  border: 1px solid black;
  margin: 0px;
  padding: 2px;
  overflow: -moz-hidden-unscrollable;
  height: 70px;
  max-height: 70px;
  line-height: 70px;
  width: 100%;
  max-width: 100%;
  background-color: white;
  text-align: center;
  font-size: 12pt;
  font-style: normal;
  text-transform: none;
  white-space : nowrap;
}

treechildren::-moz-tree-cell-text(external) {
    font-style: italic ! important;
}

/*
spinbuttons image { border: solid thin black; padding: 2px; -moz-border-radius: 3px }

spinbuttons image:hover:active {
   background-color: #90A1B3;
   color: #FFFFFF;
 }
*/